unit WABD_HTMLEditor;

{$I kbmWABD.inc}
{$IFDEF VER100}
{$ASSERTIONS ON}
{$ENDIF}

interface

uses
{$ifdef LEVEL6}
  DesignEditors, DesignIntf,
{$else}
  DsgnIntf,
{$endif}
  WABD_FormSecEditor,
  WABD_Reg,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  WABD_Objects, StdCtrls, ExtCtrls, Menus, Buttons,
  ShellAPI, ComCtrls
 ;


type
   TWABDHTMLEditor = class(TForm)
    Panel1: TPanel;
    ToolMenu: TPopupMenu;
    FlatButtons1: TMenuItem;
    btnOK: TMainMenu;
    Preview1: TMenuItem;
    HTML1: TMenuItem;
    WML1: TMenuItem;
    FSPanel: TPanel;
    Panel3: TPanel;
    Label1: TLabel;
    mHTML: TMemo;
    btnLoadHTML: TButton;
    btnSaveHTML: TButton;
    Button1: TButton;
    odLoad: TOpenDialog;
    sdSave: TSaveDialog;
    btnEditHTML: TButton;
    btnEditWML: TButton;
    PageControl1: TPageControl;
    tsElements: TTabSheet;
    TopButPanel: TPanel;
    AnchorSpeed: TSpeedButton;
    ImageSpeed: TSpeedButton;
    LabelSpeed: TSpeedButton;
    MemoSpeed: TSpeedButton;
    ButtonSpeed: TSpeedButton;
    EditSpeed: TSpeedButton;
    ComboSpeed: TSpeedButton;
    RadioSpeed: TSpeedButton;
    ListSpeed: TSpeedButton;
    CheckSpeed: TSpeedButton;
    LiveImageSpeed: TSpeedButton;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    HiddenSpeed: TSpeedButton;
    Panel2: TPanel;
    Label2: TLabel;
    ObjectCombo: TListBox;
    DeleteObjectBut: TButton;
    btnGenHTML: TButton;
    btnGenWML: TButton;
    btnElemTemplate: TButton;
    btnDeleteall: TButton;
    Label4: TLabel;
    btnAnalyze: TButton;
    Label5: TLabel;
    btnAnalyzeModify: TButton;
    Label6: TLabel;
    mLog: TMemo;
    Label3: TLabel;
    SpeedButton4: TSpeedButton;
    SpeedButton5: TSpeedButton;
    tsStdTemplates: TTabSheet;
    btnInsertStdTemplate: TButton;
    lbStdTemplates: TListBox;
    Label7: TLabel;
    ReplaceDialog1: TReplaceDialog;
    btnCut: TButton;
    btnCopy: TButton;
    btnPaste: TButton;
    dlgFind: TFindDialog;
    procedure ObjectComboChange(Sender: TObject);
    procedure DeleteObjectButClick(Sender: TObject);
    procedure FlatButtons1Click(Sender: TObject);
    procedure ObjectSpeedClick(Sender: TObject);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure FormDestroy(Sender: TObject);
    procedure HTML1Click(Sender: TObject);
    procedure WML1Click(Sender: TObject);
    procedure btnGenHTMLClick(Sender: TObject);
    procedure btnGenWMLClick(Sender: TObject);
    procedure btnAnalyzeClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure btnElemTemplateClick(Sender: TObject);
    procedure btnDeleteallClick(Sender: TObject);
    procedure btnLoadHTMLClick(Sender: TObject);
    procedure btnSaveHTMLClick(Sender: TObject);
    procedure btnEditHTMLClick(Sender: TObject);
    procedure btnEditWMLClick(Sender: TObject);
    procedure btnInsertStdTemplateClick(Sender: TObject);
    procedure btnCutClick(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
    procedure btnPasteClick(Sender: TObject);
   private
      FWABDHTML   : TWABD_Base_HTML;
      procedure   SetWABDHTML(NewHTML: TWABD_Base_HTML);
   public
{$ifdef LEVEL6}
      MyDesigner   : IDesigner;
{$else}
 {$ifdef LEVEL5}
      MyDesigner   : IFormDesigner;
 {$else}
  {$ifdef LEVEL4}
      MyDesigner   : IFormDesigner;
  {$else}
   {$ifdef LEVEL3}
      MyDesigner   : TFormDesigner; // Delphi 3, C++ Builder 3
   {$endif}
  {$endif}
 {$endif}
{$endif}

      fs          : TWABD_FormSection;
      property    WABDHTML: TWABD_Base_HTML read FWABDHTML write SetWABDHTML;
      procedure   ReflectedNotify(Sender: TObject; AComponent: TComponent; Operation: TOperation);
      procedure   ChildNameChanged(Sender: TObject; const OldName, NewName: string);
      procedure   Initialize;
      procedure   InitializeObjects;
      function    AddObject(Index: integer): TWABD_SectionObject;
      procedure   EnableSectionButs(b: boolean);
      procedure   Preview(What:integer);
   end;

  { TWABD_HTMLParser }
  TWABD_HTMLToken = (wabd_tEnd, wabd_tSymbol, wabd_tLineEnd, wabd_tHTMLTag);

  TWABD_HTMLParser = class
  private
    FText: string;
    FSourcePtr: PChar;
    FTokenPtr: PChar;
    FTokenString: string;
    FTokenStartOfs:integer;
    FTokenEndOfs:integer;
    FToken: TWABD_HTMLToken;
    FAttribs:TStringList;
    function GetIndexOfAttrib(aAttrib:string):integer;
  public
    constructor Create(const Text: string);
    destructor Destroy; override;

    procedure NextToken(MultiLine:boolean);
    procedure BuildAttribs(Tag:string);
    function TokenHTMLTagIs(s:string):boolean;

    property TokenString:string read FTokenString;
    property TokenStartOfs:integer read FTokenStartOfs;
    property TokenEndOfs:integer read FTokenEndOfs;
    property Attribs:TStringList read FAttribs;
    property IndexOfAttrib[aAttrib:string]:integer read GetIndexOfAttrib;
  end;

var
  WABDHTMLEditor: TWABDHTMLEditor;

implementation


{$R *.DFM}

function MaxInt(a, b: integer): integer;
begin
   if a < b then Result := a
      else Result := b;
end;

procedure TWABDHTMLEditor.SetWABDHTML(NewHTML: TWABD_Base_HTML);
begin
   FWABDHTML := NewHTML;
end;

function TWABDHTMLEditor.AddObject(Index: integer): TWABD_SectionObject;
var
   newc  : TComponentClass;
   c     : TWABD_Object;
   ni    : integer;
begin
   newc := nil;
   case Index of
      0  :  newc := TWABD_Anchor;
      1  :  newc := TWABD_Image;
      2  :  newc := TWABD_Label;
      3  :  newc := TWABD_Memo;
      4  :  newc := TWABD_Button;
      5  :  newc := TWABD_Edit;
      6  :  newc := TWABD_ComboBox;
      7  :  newc := TWABD_RadioButton;
      8  :  newc := TWABD_ListBox;
      9  :  newc := TWABD_CheckBox;
      10 :  newc := TWABD_LiveImage;
      11 :  newc := TWABD_Chart;
      12 :  newc := TWABD_HTMLEmbed;
      13 :  newc := TWABD_HTMLFileEmbed;
      14 :  newc := TWABD_UploadFile;
      15 :  newc := TWABD_Hidden;
   end;
   Result := nil;
   if newc = nil then exit;

   c := MyDesigner.CreateComponent(newc, WABDHTML, 0, 0, 0, 0) as TWABD_Object;
   Assert(c<>nil, 'c = nil');
   c.Parent := WABDHTML;
   MyDesigner.Modified;
   ni := ObjectCombo.Items.Add(c.Name);
   ObjectCombo.ItemIndex := ni;
   ObjectComboChange(Self);
   Result := c as TWABD_SectionObject;
end;

procedure TWABDHTMLEditor.Initialize;
begin
   Caption := 'WABD Template Designer - ' + WABDHTML.Name;
   Assert(MyDesigner<>nil, 'Designer = nil');

   WABDHTML.RefNotify := ReflectedNotify;
   WABDHTML.OnChildNameChanged := ChildNameChanged;

   // WABDForm.FreeNotification(Self);
   ObjectCombo.Items.Clear;
   ObjectComboChange(Self);
   InitializeObjects;

   mHTML.Text:=WABDHTML.HTML.Text;
end;

procedure TWABDHTMLEditor.InitializeObjects;
var
   i : integer;
   c : TWABD_Object;
begin
   ObjectCombo.Items.Clear;

   for i := 0 to WABDHTML.ChildCount-1 do begin
      c := WABDHTML.Children[i];
      Assert(c<>nil, 'c = nil');
      ObjectCombo.Items.Add(c.Name);
   end;
end;


procedure TWABDHTMLEditor.ObjectComboChange(Sender: TObject);
var
   ni : integer;
   c  : TWABD_Object;
begin
   ni := ObjectCombo.ItemIndex;

   DeleteObjectBut.Enabled := ni<>-1;

   if ni=-1 then exit;
   c := WABDHTML.ChildByName(ObjectCombo.Items[ni]) as TWABD_Object;
   Assert(c<>nil, 'c = nil');
   MyDesigner.SelectComponent(c);
end;

procedure TWABDHTMLEditor.DeleteObjectButClick(Sender: TObject);
var
   ni : integer;
   c  : TWABD_Object;
begin
   ni := ObjectCombo.ItemIndex;

   if ni=-1 then exit;
   c := WABDHTML.ChildByName(ObjectCombo.Items[ni]) as TWABD_Object;
   Assert(c<>nil, 'c = nil');

   c.Free;
   MyDesigner.SelectComponent(nil);
   // SectionCombo.Items.Delete(ni); // No need for this, ReflectedNotify removes it automatically
   ni := MaxInt(ni, ObjectCombo.Items.Count-1);
   ObjectCombo.ItemIndex := ni;
   if ObjectCombo.Items.Count=0 then ObjectCombo.Items.Clear;  // ComboBox Bug
   ObjectComboChange(Self);
end;

// I had to do this hack because Delphi 3 won't let me compile with
// the DsgnWnds unit.
procedure TWABDHTMLEditor.ReflectedNotify(Sender: TObject; AComponent: TComponent; Operation: TOperation);
var
   i : integer;
begin
   if (Operation <> opRemove) then exit;

   if AComponent=WABDHTML then begin
      WABDHTML.RefNotify := nil;
      WABDHTML := nil;
      Visible  := False;
   end;

   i := ObjectCombo.Items.IndexOf(AComponent.Name);
   if i<>-1 then begin
      ObjectCombo.Items.Delete(i);
      if ObjectCombo.Items.Count=0 then ObjectCombo.Items.Clear;  // ComboBox Bug
      ObjectComboChange(Self);
   end;
end;



procedure TWABDHTMLEditor.FlatButtons1Click(Sender: TObject);
var
   i : integer;
   c : TControl;
begin
   FlatButtons1.Checked := not FlatButtons1.Checked;

   for i := 0 to TopButPanel.ControlCount-1 do begin
      c := TopButPanel.Controls[i];
      if (c is TSpeedButton) then begin
         {$IFDEF VER100}
         (c as TSpeedButton).Flat := FlatButtons1.Checked;
         {$ENDIF}
      end;
   end;
end;

procedure TWABDHTMLEditor.ObjectSpeedClick(Sender: TObject);
begin
     AddObject((Sender as TSpeedButton).Tag);
end;

procedure TWABDHTMLEditor.EnableSectionButs(b: boolean);
var
   i : integer;
   c : TControl;
begin
   for i := 0 to TopButPanel.ControlCount-1 do begin
      c := TopButPanel.Controls[i];
      if (c is TSpeedButton) then
         (c as TSpeedButton).Enabled := b;
   end;
end;

procedure TWABDHTMLEditor.Preview(What:integer);
var
   f    : string;
   h    : string;
   fstm : TFileStream;
   rc   : integer;
begin
   case What of
     1: begin
             h := mHTML.Text;
             f := 'PreviewTemp.wml';
        end;
     else
        begin
             h := mHTML.Text;
             f := 'PreviewTemp.htm';
        end;
   end;
   fstm := TFileStream.Create(f, fmCreate or fmOpenWrite or fmShareDenyWrite);
   fstm.Write(h[1], Length(h));
   fstm.Free;

   rc := ShellExecute(Handle, 'open', PChar(f), nil, nil, SW_SHOWNORMAL);
   if (rc <= 32) then ShowMessage('Unable to execute: ' + f + #13#10 + 'Browser not installed?');
end;

procedure TWABDHTMLEditor.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if (Operation = opRemove) and (AComponent = WABDHTML) then begin
      WABDHTML := nil;
      Hide;
   end;
end;

procedure TWABDHTMLEditor.FormDestroy(Sender: TObject);
begin
     if WABDHTML<>nil then WABDHTML.RefNotify := nil;
end;

procedure TWABDHTMLEditor.ChildNameChanged(Sender: TObject; const OldName, NewName: string);
var
   i : integer;
begin
   i := ObjectCombo.Items.IndexOf(OldName);
   if i<>-1 then begin
      ObjectCombo.Items[i] := NewName;
   end;
end;

procedure TWABDHTMLEditor.HTML1Click(Sender: TObject);
begin
     Preview(0);
end;

procedure TWABDHTMLEditor.WML1Click(Sender: TObject);
begin
     Preview(1);
end;

procedure TWABDHTMLEditor.btnGenHTMLClick(Sender: TObject);
var
   ni:integer;
   c:TWABD_Object;
   s:string;
   p:TPoint;
begin
     ni := ObjectCombo.ItemIndex;
     if ni=-1 then exit;
     c := WABDHTML.ChildByName(ObjectCombo.Items[ni]) as TWABD_Object;
     Assert(c<>nil, 'c = nil');
     p:=mHTML.CaretPos;
     s:=mHTML.Lines[p.y];
     mHTML.Lines[p.y] := copy(s,1,p.x)+c.Object_To_HTML+Copy(s,p.x+1,length(s));
end;

procedure TWABDHTMLEditor.btnGenWMLClick(Sender: TObject);
var
   ni:integer;
   c:TWABD_Object;
   s:string;
   p:TPoint;
begin
     ni := ObjectCombo.ItemIndex;
     if ni=-1 then exit;
     c := WABDHTML.ChildByName(ObjectCombo.Items[ni]) as TWABD_Object;
     Assert(c<>nil, 'c = nil');
     p:=mHTML.CaretPos;
     s:=mHTML.Lines[p.y];
     mHTML.Lines[p.y] := copy(s,1,p.x)+c.Object_To_WML+Copy(s,p.x+1,length(s));
end;

procedure TWABDHTMLEditor.btnAnalyzeClick(Sender: TObject);
var
   i:integer;
   lastendofs,endofs,startofs:integer;
   parser:TWABD_HTMLParser;
   lastlistobject:TWABD_SelLinesObject;
   lst:TStringList;
   s:string;
   sHTML,sNewHTML:string;
   bChecked:boolean;
   sName,sType,sValue,sCaption:string;
   c:TWABD_Object;
   cc:TComponentClass;
   DoModifyHTML:boolean;
   pc:PChar;
begin
     DoModifyHTML:=(Sender=btnAnalyzeModify);

     mLog.Clear;
     sHTML:=mHTML.Text;
     sNewHTML:='';
     lastlistobject:=nil;
     parser := TWABD_HTMLParser.Create(sHTML);
     lst:=TStringList.Create;
     try
        lastendofs:=1;
        while parser.FToken <> wabd_tEnd do
        begin
             startofs:=parser.TokenStartOfs;
             endofs:=parser.TokenEndOfs;
             case parser.FToken of
                  wabd_tHTMLTag:
                    begin
                         sType:='';
                         sName:='';
                         sValue:='';
                         sCaption:='';
                         bChecked:=false;
                         if parser.TokenHTMLTagIs('INPUT') then
                         begin
                              parser.BuildAttribs(parser.TokenString);

                              i:=parser.IndexOfAttrib['NAME'];
                              if (i>=0) and (i<parser.Attribs.Count-1) then sName:=parser.Attribs[i+1];

                              i:=parser.IndexOfAttrib['TYPE'];
                              if (i>=0) and (i<parser.Attribs.Count-1) then sType:=parser.Attribs[i+1];

                              i:=parser.IndexOfAttrib['VALUE'];
                              if (i>=0) and (i<parser.Attribs.Count-1) then sValue:=parser.Attribs[i+1];
                         end
                         else if parser.TokenHTMLTagIs('TEXTAREA') then
                         begin
                              parser.BuildAttribs(parser.TokenString);

                              i:=parser.IndexOfAttrib['NAME'];
                              if (i>=0) and (i<parser.Attribs.Count-1) then sName:=parser.Attribs[i+1];
                              sType:='TEXTAREA';
                         end
                         else if parser.TokenHTMLTagIs('OPTION') then
                         begin
                              parser.BuildAttribs(parser.TokenString);
                              i:=parser.IndexOfAttrib['VALUE'];
                              if (i>=0) and (i<parser.Attribs.Count-1) then sValue:=parser.Attribs[i+1];
                              sType:='OPTION';
                         end
                         else if parser.TokenHTMLTagIs('SELECT') then
                         begin
                              parser.BuildAttribs(parser.TokenString);

                              i:=parser.IndexOfAttrib['NAME'];
                              if (i>=0) and (i<parser.Attribs.Count-1) then sName:=parser.Attribs[i+1];

                              i:=parser.IndexOfAttrib['SIZE'];
                              if (i>=0) and (i<parser.Attribs.Count-1) then
                              begin
                                   if parser.Attribs[i+1]='1' then
                                      sType:='COMBOBOX'
                                   else
                                       sType:='LISTBOX';
                              end
                              else
                                  sType:='LISTBOX';
                         end;

                         sType:=UpperCase(sType);

//                         mLog.Lines.Add('Type:'+sType+': Name='+sName+' ***** '+parser.TokenString);

                         // Determine component type.
                         cc:=nil;
                         if (stype='TEXT') or (stype='PASSWORD') then
                            cc:=TWABD_Edit
                         else if stype='HIDDEN' then
                            cc:=TWABD_Hidden
                         else if stype='RADIO' then
                         begin
                              cc:=TWABD_RadioButton;
                              s:=sName;
                              sName:=sValue;
                              sValue:=s;

                              // Get Checked.
                              i:=parser.IndexOfAttrib['CHECKED'];
                              bChecked:=(i>=0) and (i<parser.Attribs.Count);

                              // Check if sName is valid. Otherwise generate new sName.
                              if not IsValidIdent(sName) then sName:=sValue+'_'+sName;

                              // Get caption of RADIO.
                              pc:=parser.FSourcePtr;
                              parser.NextToken(false);
                              if (parser.FToken=wabd_tSymbol) then
                              begin
                                   sCaption:=parser.TokenString;
                                   endofs:=parser.TokenEndOfs;
                              end
                              else
                                  parser.FSourcePtr:=pc;
                         end
                         else if stype='CHECKBOX' then
                         begin
                              cc:=TWABD_CheckBox;

                              // Get Checked.
                              i:=parser.IndexOfAttrib['CHECKED'];
                              bChecked:=(i>=0) and (i<parser.Attribs.Count);

                              // Get caption of CHECKBOX.
                              pc:=parser.FSourcePtr;
                              parser.NextToken(false);
                              if (parser.FToken=wabd_tSymbol) then
                              begin
                                   sCaption:=parser.TokenString;
                                   endofs:=parser.TokenEndOfs;
                              end
                              else
                                  parser.FSourcePtr:=pc;
                         end
                         else if stype='FILE' then
                            cc:=TWABD_UploadFile
                         else if stype='IMAGE' then
                            cc:=TWABD_Image
                         else if (stype='SUBMIT') or (stype='BUTTON') then
                            cc:=TWABD_Button
                         else if stype='COMBOBOX' then
                            cc:=TWABD_ComboBox
                         else if stype='LISTBOX' then
                            cc:=TWABD_ListBox
                         else if stype='TEXTAREA' then
                         begin
                              cc:=TWABD_Memo;

                              // Get caption of TEXTAREA.
                              pc:=parser.FSourcePtr;
                              parser.NextToken(true);
                              if (parser.FToken=wabd_tSymbol) then
                              begin
                                   sCaption:=parser.TokenString;
                                   endofs:=parser.TokenEndOfs;
                              end
                              else
                                  parser.FSourcePtr:=pc;
                         end
                         else if stype='/SELECT' then
                         begin
                              if lastlistobject<>nil then lastendofs:=endofs;
                         end
                         else if stype='OPTION' then
                         begin
                              // Get caption of OPTION.
                              pc:=parser.FSourcePtr;
                              parser.NextToken(false);
                              if (parser.FToken=wabd_tSymbol) then
                              begin
                                   sCaption:=parser.TokenString;
                                   endofs:=parser.TokenEndOfs;
                              end
                              else
                                  parser.FSourcePtr:=pc;

                              if sCaption<>'' then
                                 sValue:=sValue+'='+sCaption;
                              if lastlistobject<>nil then
                              begin
                                   lastlistobject.Lines.Add(sValue);
                                   lastendofs:=endofs;
                              end;
                         end;

                         if cc<>nil then
                         begin
                              // Check if component already existing.
                              if sName<>'' then
                              begin
                                   c:=WABDHTML.ChildByName(sName);

                                   if c<>nil then
                                      mLog.Lines.Add(sName+' already existing. Will not add.')
                                   else
                                   begin
                                        mLog.Lines.Add('Adding '+sName+' of type '+cc.ClassName);

                                        c:=MyDesigner.CreateComponent(cc,WABDHTML,0,0,0,0) as TWABD_Object;
                                        Assert(c<>nil, 'c = nil');

                                        if (c is TWABD_RadioButton) then
                                        begin
                                             if (sValue<>'') then TWABD_RadioButton(c).GroupName:=sValue;
                                             TWABD_RadioButton(c).Caption:=sCaption;
                                             TWABD_RadioButton(c).Checked:=bChecked;
                                        end
                                        else if (c is TWABD_CheckBox) then
                                        begin
                                             TWABD_CheckBox(c).Caption:=sCaption;
                                             TWABD_CheckBox(c).Checked:=bChecked;
                                        end
                                        else if (c is TWABD_Memo) then
                                             TWABD_Memo(c).Lines.Text:=sCaption
                                        else if (c is TWABD_Edit) then
                                        begin
                                             TWABD_Edit(c).Text:=sValue;
                                             if sType='PASSWORD' then
                                                TWABD_Edit(c).IsPassword:=true;
                                        end
                                        else if (c is TWABD_SelLinesObject) then
                                             lastlistobject:=TWABD_SelLinesObject(c);

                                        if sName<>'' then c.Name:=sName;
                                        c.Parent := WABDHTML;
                                        MyDesigner.Modified;
                                        i:=ObjectCombo.Items.Add(c.Name);

                                        // Clear out list/combo boxes.
                                        if (c is TWABD_SelLinesObject) then
                                           TWABD_SelLinesObject(c).Lines.Clear;

                                        ObjectCombo.ItemIndex:=i;
                                        ObjectComboChange(Self);
                                   end;

                                   // Check if to modify HTML.
                                   if DoModifyHTML then
                                   begin
                                        s:='[!--TEMPLATE '+c.Name+'--!]';
                                        mLog.Lines.Add('Modifying to '+s);
                                        sNewHTML:=sNewHTML+copy(sHTML,lastendofs,startofs-lastendofs-1)+s;
                                   end;
                              end
                              else if DoModifyHTML then
                                      sNewHTML:=sNewHTML+copy(sHTML,lastendofs,endofs-lastendofs);
                         end
                         else if DoModifyHTML then
                                 sNewHTML:=sNewHTML+copy(sHTML,lastendofs,endofs-lastendofs);
                    end;
                  wabd_tSymbol:
                    if DoModifyHTML then
                       sNewHTML:=sNewHTML+copy(sHTML,lastendofs,endofs-lastendofs);
             end;
             lastendofs:=parser.TokenEndOfs;
             parser.NextToken(true);
        end;
     finally
        parser.free;
        lst.free;

        if DoModifyHTML then
           mHTML.Text:=sNewHTML;
     end;
end;

procedure TWABDHTMLEditor.Button1Click(Sender: TObject);
begin
     WABDHTML.HTML.Text:=mHTML.Text;
     MyDesigner.Modified;
     Close;
end;

procedure TWABDHTMLEditor.btnElemTemplateClick(Sender: TObject);
var
   ni:integer;
   c:TWABD_Object;
   s:string;
   p:TPoint;
begin
     ni := ObjectCombo.ItemIndex;
     if ni=-1 then exit;
     c := WABDHTML.ChildByName(ObjectCombo.Items[ni]) as TWABD_Object;
     Assert(c<>nil, 'c = nil');
     p:=mHTML.CaretPos;
     s:=mHTML.Lines[p.y];
     mHTML.Lines[p.y] := copy(s,1,p.x)+'[!--TEMPLATE '+c.Name+'--!]'+Copy(s,p.x+1,length(s));
end;

//*******************************************************
// TSimpleHTMLParser
//*******************************************************

constructor TWABD_HTMLParser.Create(const Text: string);
begin
     FText := Text;
     FSourcePtr := PChar(Text);
     FAttribs:=TStringList.Create;
     NextToken(true);
end;

destructor TWABD_HTMLParser.Destroy;
begin
     FAttribs.Free;
     inherited;
end;

procedure TWABD_HTMLParser.NextToken(MultiLine:boolean);
var
   P, TokenStart: PChar;
begin
     FTokenString := '';
     P := FSourcePtr;
     while (P^ <> #0) and (P^ <= ' ') do Inc(P);
     FTokenPtr := P;
     TokenStart:=P;
     case P^ of
       '<':
          begin
               Inc(P);
               TokenStart := P;
               while (P^ <> '>') and (P^ <> #0) do Inc(P);
               SetString(FTokenString, TokenStart, P - TokenStart);
               FToken := wabd_tHTMLTag;
               Inc(P);
          end;
       #0:
          FToken := wabd_tEnd;
       else
          begin
               TokenStart := P;
               Inc(P);
               while not (P^ in ['<', #0]) do
               begin
                    if (p^ in [#10,#13]) and (not Multiline) then break;
                    Inc(P);
               end;
               SetString(FTokenString, TokenStart, P - TokenStart);
               FToken := wabd_tSymbol;
          end;
     end;
     FTokenStartOfs:=TokenStart-PChar(FText)+1;
     FSourcePtr := P;
     FTokenEndOfs:=FSourcePtr-PChar(FText)+1;
end;

function TWABD_HTMLParser.TokenHTMLTagIs(s:string):boolean;
var
   s1:string;
begin
     s1:=UpperCase(FTokenString);
     s:=UpperCase(s);
     Result := (FToken = wabd_tHTMLTag) and (copy(s1,1,length(s))=s);
end;

procedure TWABD_HTMLParser.BuildAttribs(Tag:string);
var
   p,l:integer;
   pStart:integer;
   str:string;
   bColSingleString,bColDblString,bColIdent:boolean;
begin
     FAttribs.Clear;

     bColSingleString:=false;
     bColDblString:=false;
     bColIdent:=false;
     pStart:=1;
     p:=1;
     l:=length(Tag);
     while p<=l do
     begin
          if (Tag[p]='''') and (not bColDblString) then
          begin
               if bColIdent then
               begin
                    str:=copy(Tag,pStart,p-pStart);
                    FAttribs.add(str);
                    bColIdent:=false;
               end;

               bColSingleString:=not bColSingleString;
               if not bColSingleString then
               begin
                    str:=copy(Tag,pStart,p-pStart);
                    FAttribs.add(str);
               end;
               pStart:=p+1;
               inc(p);
               continue;
          end
          else if (Tag[p]='"') and (not bColSingleString) then
          begin
               if bColIdent then
               begin
                    str:=copy(Tag,pStart,p-pStart);
                    FAttribs.add(str);
                    bColIdent:=false;
               end;

               bColDblString:=not bColDblString;
               if not bColDblString then
               begin
                    str:=copy(Tag,pStart,p-pStart);
                    FAttribs.add(str);
               end;
               pStart:=p+1;
               inc(p);
               continue;
          end;

          if bColDblString or bColSingleString then
          begin
               inc(p);
               continue;
          end;

          if (Tag[p] in [' ','=']) then
          begin
               if bColIdent then
               begin
                    str:=trim(copy(Tag,pStart,p-pStart));
                    FAttribs.add(str);
                    bColIdent:=false;
               end;
               pStart:=p+1;
               inc(p);
               continue;
          end
          else if not bColIdent then
          begin
               bColIdent:=true;
               pStart:=p;
          end;
          inc(p);
     end;

     if (p>pStart) then
     begin
          str:=copy(Tag,pStart,p-pStart);
          FAttribs.add(str);
     end;
end;

function TWABD_HTMLParser.GetIndexOfAttrib(aAttrib:string):integer;
var
   i:integer;
   s:string;
begin
     s:=Uppercase(aAttrib);
     Result:=-1;
     for i:=0 to FAttribs.count-1 do
         if UpperCase(FAttribs.Strings[i])=s then
         begin
              Result:=i;
              exit;
         end;
end;

procedure TWABDHTMLEditor.btnDeleteallClick(Sender: TObject);
var
   i:integer;
   c :TWABD_Object;
begin
     MyDesigner.SelectComponent(nil);
     for i:=ObjectCombo.Items.Count-1 downto 0 do
     begin
          c:=WABDHTML.ChildByName(ObjectCombo.Items[i]) as TWABD_Object;
          Assert(c<>nil, 'c = nil');

          c.Free;
     end;

     ObjectCombo.Items.Clear;
     ObjectComboChange(Self);
     MyDesigner.Modified;
end;

procedure TWABDHTMLEditor.btnLoadHTMLClick(Sender: TObject);
begin
     if odLoad.Execute then
        mHTML.Lines.LoadFromFile(odLoad.Filename);

end;

procedure TWABDHTMLEditor.btnSaveHTMLClick(Sender: TObject);
begin
     if sdSave.Execute then
        mHTML.Lines.SaveToFile(sdSave.Filename);
end;

procedure TWABDHTMLEditor.btnEditHTMLClick(Sender: TObject);
var
   fn:string;
begin
     fn:=mHTML.Name+'.html';
     mHTML.Lines.SaveToFile(fn);
     ShellExecute(handle, 'edit', PChar(fn), nil, nil, SW_SHOWNORMAL);
end;

procedure TWABDHTMLEditor.btnEditWMLClick(Sender: TObject);
var
   fn:string;
begin
     fn:=mHTML.Name+'.wml';
     mHTML.Lines.SaveToFile(fn);
     ShellExecute(handle, 'edit', PChar(fn), nil, nil, SW_SHOWNORMAL);
end;

procedure TWABDHTMLEditor.btnInsertStdTemplateClick(Sender: TObject);
var
   ni:integer;
   c:TWABD_Object;
   s:string;
   p:TPoint;
begin
     ni := lbStdTemplates.ItemIndex;
     if ni=-1 then exit;
     p:=mHTML.CaretPos;
     s:=mHTML.Lines[p.y];
     mHTML.Lines[p.y] := copy(s,1,p.x)+lbStdTemplates.Items[ni]+Copy(s,p.x+1,length(s));
end;

procedure TWABDHTMLEditor.btnCutClick(Sender: TObject);
begin
     mHTML.CutToClipboard;
end;

procedure TWABDHTMLEditor.btnCopyClick(Sender: TObject);
begin
     mHTML.CopyToClipboard;
end;

procedure TWABDHTMLEditor.btnPasteClick(Sender: TObject);
begin
     mHTML.PasteFromClipboard;
end;

end.
